package HLstats_Player;
#
# HLstatsX - Real-time player and clan rankings and statistics for Half-Life:Source
# http://www.hlstatsx.com/
# Copyright (C) 2005 Tobias Oetzel (Tobi@hlstatsx.com)
#
# HlstatsX is an enhanced version of HLstats made by Simon Garner
# HLstats - Real-time player and clan rankings and statistics for Half-Life
# http://sourceforge.net/projects/hlstats/
# Copyright (C) 2001  Simon Garner
#             
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

#
# Constructor
#

sub new
{
	my $class_name = shift;
	my %params = @_;
	
	my $self = {};
	bless($self, $class_name);
	
	# Initialise Properties
	$self->{userid}            = 0;
	$self->{server}            = "";
	$self->{server_id}         = 1;
	$self->{name}              = "";
	$self->{uniqueid}          = "";
	$self->{plain_uniqueid}    = "";
	$self->{address}           = "";
	$self->{cli_port}          = "";
	$self->{ping}              = 0;
	$self->{connect_time}      = 0;
	
	$self->{playerid}          = 0;
	$self->{clan}              = 0;
	$self->{kills}             = 0;
	$self->{deaths}            = 0;
	$self->{suicides}          = 0;
	$self->{skill}             = 1000;
	$self->{game}              = 0;
	$self->{team}              = "";
	$self->{role}              = "";
	$self->{timestamp}         = 0;
	$self->{headshots}         = 0;
	$self->{shots}             = 0;
	$self->{hits}              = 0;
	
	$self->{auto_command}      = "";
	$self->{auto_type}         = "";
	$self->{auto_time}         = 0;
	$self->{auto_time_count}   = 0;
	
	$self->{session_skill}     = 0;
	$self->{session_kills}     = 0;
	$self->{session_deaths}    = 0;
	$self->{session_suicides}  = 0;
	$self->{session_headshots} = 0;
	$self->{session_shots}     = 0;
	$self->{session_hits}      = 0;

	$self->{map_kills}         = 0;
	$self->{map_deaths}        = 0;
	$self->{map_suicides}      = 0;
	$self->{map_headshots}     = 0;
	$self->{map_shots}         = 0;
	$self->{map_hits}          = 0;
	$self->{is_dead}           = 0;
	$self->{has_bomb}          = 0;
	
	$self->{is_banned}         = 0;
	
	
	# Set Property Values
	
	die("HLstats_Player->new(): must specify player's uniqueid\n")
		unless (defined($params{uniqueid}));
	
	die("HLstats_Player->new(): must specify player's name\n")
		unless ($params{name} ne "");
	
	$self->{plain_uniqueid} = $params{uniqueid};
	$self->setUniqueId($params{uniqueid});
	$self->setName($params{name});
	
	while (my($key, $value) = each(%params))
	{
		if ($key ne "name" && $key ne "uniqueid")
		{
			$self->set($key, $value);
		}
	}
	
	$self->getAddress();
	$self->updateDB();

	&::printNotice("Created new player object " . $self->getInfoString());
	return $self;
}


#
# Set property 'key' to 'value'
#

sub set
{
	my ($self, $key, $value, $no_updatetime) = @_;
	
	if (defined($self->{$key}))
	{
		if ($no_updatetime == 0) {
			$self->{timestamp} = $::ev_unixtime;
		}
		
		if ($self->get($key) eq $value)
		{
			if ($g_debug > 2)
			{
				&::printNotice("Hlstats_Player->set ignored: Value of \"$key\" is already \"$value\"");
			}
			return 0;
		}
		
		if ($key eq "uniqueid")
		{
			return $self->setUniqueId($value);
		}
		elsif ($key eq "name")
		{
			return $self->setName($value);
		}
		else
		{
			$self->{$key} = $value;
			return 1;
		}
	}
	else
	{
		warn("HLstats_Player->set: \"$key\" is not a valid property name\n");
		return 0;
	}
}


#
# Increment (or decrement) the value of 'key' by 'amount' (or 1 by default)
#

sub increment
{
	my ($self, $key, $amount, $no_updatetime) = @_;
	
	$amount = int($amount);
	$amount = 1 if ($amount == 0);
	
	my $value = $self->get($key);
	$self->set($key, $value + $amount, $no_updatetime);
}


#
# Get value of property 'key'
#

sub get
{
	my ($self, $key) = @_;
	
	if (defined($self->{$key}))
	{
		return $self->{$key};
	}
	else
	{
		warn("HLstats_Player->get: \"$key\" is not a valid property name\n");
	}
}


#
# Set player's uniqueid
#

sub setUniqueId
{
	my ($self, $uniqueid) = @_;
	
	my $playerid = &::getPlayerId($uniqueid);
	
	if ($playerid)
	{
		# An existing player. Get their skill rating.
		
		my $query = "
			SELECT
				skill
			FROM
				hlstats_Players
			WHERE
				playerId='$playerid'
		";
		my $result = &::doQuery($query);
		($self->{skill}) = $result->fetchrow_array;
		$result->finish;
	}
	else
	{
		# This is a new player. Create a new record for them in the Players
		# table.
		
		my $query = "
			INSERT INTO
				hlstats_Players
				(
					lastName,
					clan,
					game
				)
			VALUES
			(
				'" . &::quoteSQL($self->get("name")) . "',
				'" . $self->get("clan") . "',
				'" . $::g_servers{$::s_addr}->{game} . "'
			)
		";
		my $result = &::doQuery($query);
		$result->finish;
		
		$result = &::doQuery("SELECT LAST_INSERT_ID()");
		($playerid) = $result->fetchrow_array;
		$result->finish;
		
		if ($playerid)
		{
			$query = "
				INSERT INTO
					hlstats_PlayerUniqueIds
					(
						playerId,
						uniqueId,
						game
					)
				VALUES
				(
					'" . $playerid . "',
					'" . &::quoteSQL($uniqueid) . "',
					'" . $::g_servers{$::s_addr}->{game} . "'
				)
			";
			$result = &::doQuery($query);
			$result->finish;
		}
		else
		{
			error("Unable to create player:\n$query");
		}
	}
	
	$self->{uniqueid} = $uniqueid;
	$self->{playerid} = $playerid;
	
	return 1;
}



#
# Set player's name
#

sub setName
{
	my ($self, $name) = @_;
	
	my $oldname = $self->get("name");

	if ($oldname eq $name)
	{
		return 2;
	}
	
	if ($oldname)
	{
		$self->updateDB();
	}
	
	$self->{name} = $name;
	$self->{clan} = &::getClanId($name);
	
	my $playerid = $self->get("playerid");
	
	if ($playerid)
	{
		my $query = "
			SELECT
				playerId
			FROM
				hlstats_PlayerNames
			WHERE
				playerId='" . $playerid . "'
				AND name='" . &::quoteSQL($self->get("name")) . "'
		";
		my $result = &::doQuery($query);
		
		if ($result->rows < 1)
		{
			$query = "
				INSERT INTO
					hlstats_PlayerNames
					(
						playerId,
						name,
						lastuse,
						numuses
					)
				VALUES
				(
					'" . $playerid . "',
					'" . &::quoteSQL($self->get("name")) . "',
					" . $::ev_datetime . ",
					1
				)
			";
			&::doQuery($query);
		}
		else
		{
			$query = "
				UPDATE
					hlstats_PlayerNames
				SET
					lastuse=" . $::ev_datetime . ",
					numuses=numuses+1
				WHERE
					playerId='" . $playerid . "'
					AND name='" . &::quoteSQL($self->get("name")) . "'
			";
			&::doQuery($query);
		}
		
		$result->finish;
	}
	else
	{
		&::error("HLstats_Player->setName(): No playerid");
	}
}



#
# Update player information in database
#

sub updateDB
{
	my ($self, $leaveLastUse, $callref) = @_;
	
	my $playerid  = $self->get("playerid");
	my $serverid  = $self->get("server_id");
	my $name      = $self->get("name");
	my $clan      = $self->get("clan");
	my $kills     = $self->get("kills");
	my $deaths    = $self->get("deaths");
	my $suicides  = $self->get("suicides");
	my $skill     = $self->get("skill");
	my $headshots = $self->get("headshots");
	my $shots     = $self->get("shots");
	my $hits      = $self->get("hits");
	
	my $team          = $self->get("team");
	my $map_kills     = $self->get("map_kills");
	my $map_deaths    = $self->get("map_deaths");
	my $map_suicides  = $self->get("map_suicides");
	my $map_headshots = $self->get("map_headshots");
	my $map_shots     = $self->get("map_shots");
	my $map_hits      = $self->get("map_hits");
	my $steamid       = $self->get("plain_uniqueid");
	
	my $is_dead       = $self->get("is_dead");
	my $has_bomb      = $self->get("has_bomb");
	my $ping          = $self->get("ping");
	my $connected     = $self->get("connect_time");
	my $skill_change  = $self->get("session_skill");
	

	unless ($playerid)
	{
		warn ("Player->Update() with no playerid set!\n");
		return 0;
	}
	
	# Update player details
	my $query = "
		UPDATE
			hlstats_Players
		SET
			lastName='" . &::quoteSQL($name) . "',
			clan='$clan',
			kills=kills + $kills,
			deaths=deaths + $deaths,
			suicides=suicides + $suicides,
			skill=$skill,
			headshots=headshots + $headshots,
			shots=shots + $shots,
			hits=hits + $hits,
			last_event=".time()."
		WHERE
			playerId='$playerid'
	";
	&::doQuery($query, "Player->updateDB(): $callref");
	
	if ($name)
	{
		# Update alias details
		$query = "
			UPDATE
				hlstats_PlayerNames
			SET
				kills=kills + $kills,
				deaths=deaths + $deaths,
				suicides=suicides + $suicides,
   			    headshots=headshots + $headshots,
    			shots=shots + $shots,
   			    hits=hits + $hits"
		;
		
		unless ($leaveLastUse)
		{
			# except on ChangeName we update the last use on a player's old name
			
			$query .= ",
				lastuse=" . $::ev_datetime . ""
			;
		}
		
		$query .= "
			WHERE
				playerId='" . $playerid . "'
				AND name='" . &::quoteSQL($self->get("name")) . "'
		";
		&::doQuery($query);
	}
	
	# reset player stat properties
	$self->set("kills",     0, 1);
	$self->set("deaths",    0, 1);
	$self->set("suicides",  0, 1);
	$self->set("headshots", 0, 1);
	$self->set("shots",     0, 1);
	$self->set("hits",      0, 1);
	
	# Update live stats
	my $query = "
		REPLACE INTO
			hlstats_Livestats
		SET
		    player_id=$playerid,
		    server_id=$serverid,
	    	steam_id='$steamid',
			name='" . &::quoteSQL($name) . "',
			team='$team',
			kills=$map_kills,
			deaths=$map_deaths,
			suicides=$map_suicides,
			headshots=$map_headshots,
			shots=$map_shots,
			hits=$map_hits,
			is_dead=$is_dead,
			has_bomb=$has_bomb,
			ping=$ping,
			connected=$connected,
			skill_change=$skill_change,
			skill=$skill
	";
	&::doQuery($query);

	
	&::printNotice("Updated player object " . $self->getInfoString());
	
	return 1;
}


#
# Update player timestamp (time of last event for player - used to detect idle
# players)
#

sub updateTimestamp
{
	my ($self, $timestamp) = @_;
	
	$timestamp = $::ev_unixtime
		unless ($timestamp);
	
	$self->{timestamp} = $timestamp;
	
	return $timestamp;
}



sub deleteLivestats
{
	my ($self) = @_;
	my $playerid = $self->get("playerid");

	# delete live stats
	my $query = "DELETE FROM hlstats_Livestats WHERE player_id=$playerid";
	&::doQuery($query);
}


#
# Returns a string of information about the player.
#

sub getInfoString
{
	my ($self) = @_;
	
	my $name = $self->get("name");
	my $playerid = $self->get("playerid");
	my $userid   = $self->get("userid");
	my $uniqueid = $self->get("uniqueid");
	my $team = $self->get("team");

	return "\"$name\" \<P:$playerid,U:$userid,W:$uniqueid,T:$team\>";
}


sub getAddress
{
	my ($self) = @_;
	
	$p_address = $self->get("address");
	if (!$p_address)
	{
  	  &::printNotice("rcon_getaddress");
	  $server_address = $self->get("server");
	  my $result = $::g_servers{$server_address}->rcon_getaddress($self->get("plain_uniqueid"));
      if ($result) {
        $self->{address}  = $result->{Address};
        $self->{cli_port} = $result->{ClientPort};
        $self->{ping}     = $result->{Ping};
        &::printEvent("RCON", "Got Address $self->{address} for Player $self->{name}", 1);
      } 
	  &::printNotice("rcon_getaddress successfully");
	}  
    return 1;
}


sub getRank
{
	my ($self, $game) = @_;
	
    my $p_steamid  = $self->get("plain_uniqueid");
    my $result = &::doQuery("
            SELECT
                  skill
            FROM
                  hlstats_Players
            WHERE
                  playerId='" . $self->get("playerid") . "'
            ");
    my ($skill) = $result->fetchrow_array;

    my $result = &::doQuery("
            SELECT
               COUNT(skill)
            FROM 
               hlstats_Players
            WHERE 
   		       game='$game'
               AND skill > $skill
               AND hideranking = 0
               AND kills >= 1
               AND IF($::g_minactivity > (UNIX_TIMESTAMP() - last_event), UNIX_TIMESTAMP() - last_event, 0) > 0
            ");
     my ($ranknumber) = $result->fetchrow_array;
     $ranknumber++;

     my $result = &::doQuery("
		    SELECT
   			    playerId,
    			IFNULL(kills / deaths, '-') AS kpd
	         FROM
	            hlstats_Players
		     WHERE
			    game='$game' 
                AND skill='$skill' 
                AND hideranking = 0
         		AND kills >= 1
                AND IF($::g_minactivity > (UNIX_TIMESTAMP() - last_event), UNIX_TIMESTAMP() - last_event, 0) > 0
	  	      ORDER BY kpd DESC,  lastName ASC
    	 ");
 	while (my ($rowdata) = $result->fetchrow_array)  {
      if ($rowdata['playerId'] == $self->get("playerid")) {
        break;
      }   
	  $ranknumber++;
	}	
	
	if ($ranknumber > 0) {
  	  return $ranknumber;
  	} else {
  	  return 100000;
  	}  

}

1;